#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import, with_statement
import os
import sys
import codecs
from setuptools import setup, find_packages
import versioneer

# Change to source's directory prior to running any command
try:
    SETUP_DIRNAME = os.path.dirname(__file__)
except NameError:
    # We're most likely being frozen and __file__ triggered this NameError
    # Let's work around that
    SETUP_DIRNAME = os.path.dirname(sys.argv[0])

if SETUP_DIRNAME != '':
    os.chdir(SETUP_DIRNAME)


def read(fname):
    '''
    Read a file from the directory where setup.py resides
    '''
    file_path = os.path.join(SETUP_DIRNAME, fname)
    with codecs.open(file_path, encoding='utf-8') as rfh:
        return rfh.read()


setup(
    name='pytest-salt',
    version=versioneer.get_version(),
    author='Pedro Algarvio',
    author_email='pedro@algarvio.me',
    maintainer='Pedro Algarvio',
    maintainer_email='pedro@algarvio.me',
    license='Apache Software License 2.0',
    url='https://github.com/saltstack/pytest-salt',
    description='Pytest Salt Plugin',
    long_description=read('README.rst'),
    packages=find_packages(),
    cmdclass=versioneer.get_cmdclass(),
    install_requires=[
        'pytest >= 2.8.1',
        'pytest-tempdir',
        'pytest-helpers-namespace',
        #'Salt',
        'psutil >= 4.2.0',
    ],
    dependency_links=[
        'git+https://github.com/saltstack/salt.git@2016.3#egg=Salt'
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Testing',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: Apache Software License',
    ],
    entry_points={
        'pytest11': [
            'salt           = pytestsalt',
            'salt.config    = pytestsalt.fixtures.config',
            'salt.daemons   = pytestsalt.fixtures.daemons',
            'salt.dirs      = pytestsalt.fixtures.dirs',
            'salt.ports     = pytestsalt.fixtures.ports',
            'salt.log       = pytestsalt.fixtures.log',
            'salt.stats     = pytestsalt.fixtures.stats'
        ],
        'salt.loader': [
            'engines_dirs      = pytestsalt.salt.loader:engines_dirs',
            'log_handlers_dirs = pytestsalt.salt.loader:log_handlers_dirs'
        ]
    },
)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Pedro Algarvio (pedro@algarvio.me)`
    :copyright: Copyright 2015 by the SaltStack Team, see AUTHORS for more details.
    :license: Apache 2.0, see LICENSE for more details.


    test_salt_minion.py
    ~~~~~~~~~~~~~~~~~~~

    Test the pytest salt plugin salt minion
'''

# Import python libs
from __future__ import absolute_import


# Import pytest libs
import pytest


def test_ping(salt_call):
    assert salt_call.run_sync('test.ping', timeout=10).exitcode == 0


@pytest.mark.gen_test
def test_ping_async(salt_call):
    result = yield salt_call.run('test.ping', timeout=10)
    assert result.exitcode == 0


def test_sync(salt_call):
    assert salt_call.run_sync('saltutil.sync_all', timeout=10).exitcode == 0


@pytest.mark.gen_test
def test_sync_async(salt_call):
    result = yield salt_call.run('saltutil.sync_all', timeout=10)
    assert result.exitcode == 0



# -*- coding: utf-8 -*-
'''
PyTest Salt Plugin
'''

# Import Python libs
import re

# Import pytest salt libs
from pytestsalt._version import get_versions

# Store the version attribute
__version__ = get_versions()['version']
del get_versions

# Define __version_info__ attribute
VERSION_INFO_REGEX = re.compile(
    r'(?P<year>[\d]{4})\.(?P<month>[\d]{1,2})\.(?P<day>[\d]{1,2})'
    r'(?:\.dev0\+(?P<commits>[\d]+)\.(?:.*))?'
)
try:
    __version_info__ = tuple([int(p) for p in VERSION_INFO_REGEX.match(__version__).groups() if p])
except AttributeError:
    __version_info__ = (-1, -1, -1)
finally:
    del VERSION_INFO_REGEX



# -*- coding: utf-8 -*-
'''
    pytestsalt.fixtures.log_server_asyncio
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    AsyncIO Log Server Fixture
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import errno
import asyncio
import logging
import threading

# Import 3rd-party libs
import msgpack

log = logging.getLogger(__name__)


def log_server_asyncio(log_server_port):
    '''
    Starts a log server.
    '''
    async def read_child_processes_log_records(reader, writer):
        unpacker = msgpack.Unpacker(raw=False)
        while True:
            try:
                wire_bytes = await reader.read(1024)
                if not wire_bytes:
                    break
                try:
                    unpacker.feed(wire_bytes)
                except msgpack.exceptions.BufferFull:
                    # Start over loosing some data?!
                    unpacker = msgpack.Unpacker(raw=False)
                    unpacker.feed(wire_bytes)
                for record_dict in unpacker:
                    record = logging.makeLogRecord(record_dict)
                    logger = logging.getLogger(record.name)
                    logger.handle(record)
            except (EOFError, KeyboardInterrupt, SystemExit):
                break
            except Exception as exc:  # pylint: disable=broad-except
                log.exception(exc)

    def process_logs(port):
        loop = asyncio.new_event_loop()
        try:
            coro = asyncio.start_server(read_child_processes_log_records, host='localhost', port=port, loop=loop)
            server = loop.run_until_complete(coro)
        except OSError as err:
            if err.errno != errno.EADDRNOTAVAIL:
                # If not address not available, in case localhost cannot be resolved
                raise
            coro = asyncio.start_server(read_child_processes_log_records, host='127.0.0.1', port=port, loop=loop)
            server = loop.run_until_complete(coro)
        try:
            loop.run_forever()
        except KeyboardInterrupt:
            pass

        server.close()
        loop.run_until_complete(server.wait_closed())
        loop.close()

    process_queue_thread = threading.Thread(target=process_logs, args=(log_server_port,))
    process_queue_thread.daemon = True
    process_queue_thread.start()



# -*- coding: utf-8 -*-
'''
pytestsalt.fixtures.ports
~~~~~~~~~~~~~~~~~~~~~~~~~

Pytest salt plugin ports fixtures
'''

# Import python libs
from __future__ import absolute_import

# Import 3rd-party libs
import pytest

# Import pytestsalt libs
from pytestsalt.utils import get_unused_localhost_port


@pytest.fixture
def master_publish_port():
    '''
    Returns an unused localhost port for the master publish interface
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def session_master_publish_port():
    '''
    Returns an unused localhost port for the master publish interface
    '''
    return get_unused_localhost_port()


@pytest.fixture
def master_return_port():
    '''
    Returns an unused localhost port for the master return interface
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def session_master_return_port():
    '''
    Returns an unused localhost port for the master return interface
    '''
    return get_unused_localhost_port()


@pytest.fixture
def master_tcp_master_pub_port():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def session_master_tcp_master_pub_port():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture
def master_tcp_master_pull_port():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def session_master_tcp_master_pull_port():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture
def master_tcp_master_publish_pull():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def session_master_tcp_master_publish_pull():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture
def master_tcp_master_workers():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def session_master_tcp_master_workers():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture
def master_engine_port():
    '''
    Returns an unused localhost port for the pytest salt master engine
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def session_master_engine_port():
    '''
    Returns an unused localhost port for the pytest session salt master engine
    '''
    return get_unused_localhost_port()


@pytest.fixture
def master_of_masters_publish_port():
    '''
    Returns an unused localhost port for the master publish interface
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def session_master_of_masters_publish_port():
    '''
    Returns an unused localhost port for the master publish interface
    '''
    return get_unused_localhost_port()


@pytest.fixture
def master_of_masters_return_port():
    '''
    Returns an unused localhost port for the master return interface
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def session_master_of_masters_return_port():
    '''
    Returns an unused localhost port for the master return interface
    '''
    return get_unused_localhost_port()


@pytest.fixture
def master_of_masters_master_tcp_master_pub_port():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def session_master_of_masters_master_tcp_master_pub_port():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture
def master_of_masters_master_tcp_master_pull_port():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def session_master_of_masters_master_tcp_master_pull_port():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture
def master_of_masters_master_tcp_master_publish_pull():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def session_master_of_masters_master_tcp_master_publish_pull():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture
def master_of_masters_master_tcp_master_workers():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def session_master_of_masters_master_tcp_master_workers():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture
def master_of_masters_engine_port():
    '''
    Returns an unused localhost port for the pytest salt master engine
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def session_master_of_masters_engine_port():
    '''
    Returns an unused localhost port for the pytest session salt master engine
    '''
    return get_unused_localhost_port()


@pytest.fixture
def minion_tcp_pub_port():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def session_minion_tcp_pub_port():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture
def minion_tcp_pull_port():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def session_minion_tcp_pull_port():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture
def proxy_tcp_pub_port():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def session_proxy_tcp_pub_port():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture
def proxy_tcp_pull_port():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def session_proxy_tcp_pull_port():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture
def minion_engine_port():
    '''
    Returns an unused localhost port for the pytest salt minion engine
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def session_minion_engine_port():
    '''
    Returns an unused localhost port for the pytest session salt minion engine
    '''
    return get_unused_localhost_port()


@pytest.fixture
def secondary_minion_tcp_pub_port():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def session_secondary_minion_tcp_pub_port():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture
def secondary_minion_tcp_pull_port():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def session_secondary_minion_tcp_pull_port():
    '''
    Returns an unused localhost port
    '''
    return get_unused_localhost_port()


@pytest.fixture
def secondary_minion_engine_port():
    '''
    Returns an unused localhost port for the pytest salt secondary minion engine
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def session_secondary_minion_engine_port():
    '''
    Returns an unused localhost port for the pytest session salt secondary minion engine
    '''
    return get_unused_localhost_port()


@pytest.fixture
def proxy_engine_port():
    '''
    Returns an unused localhost port for the pytest salt proxy engine
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def session_proxy_engine_port():
    '''
    Returns an unused localhost port for the pytest session salt proxy engine
    '''
    return get_unused_localhost_port()


@pytest.fixture
def syndic_engine_port():
    '''
    Returns an unused localhost port for the pytest salt syndic engine
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def session_syndic_engine_port():
    '''
    Returns an unused localhost port for the pytest session salt syndic engine
    '''
    return get_unused_localhost_port()


@pytest.fixture
def sshd_port():
    '''
    Returns an unused localhost port to run an sshd server
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def session_sshd_port():
    '''
    Returns an unused localhost port to run a session scoped sshd server
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def log_server_port():
    '''
    Returns an unused localhost port for the pytest logging manager
    '''
    return get_unused_localhost_port()


@pytest.fixture(scope='session')
def salt_log_port(log_server_port):
    '''
    Returns the log_server_port fixture value for backwards compatibility
    '''
    return log_server_port
